﻿#pragma once

#include <cigui/core/View.hpp>
#include <SFML/Graphics/Color.hpp>

namespace sf {
	class RectangleShape;
}

namespace cig {
	struct Rectangle : View {
		sf::Color m_Color;
		sf::Color m_BorderColor;
		float m_BorderThickness = 0;

		Rectangle *setBorderColor(const sf::Color &color) {
			m_BorderColor = color;
			return this;
		}
		Rectangle *setBorderThickness(float thickness) {
			m_BorderThickness = thickness;
			return this;
		}
		Rectangle *setColor(const sf::Color &color) {
			m_Color = color;
			return this;
		}
		Rectangle *setSize(const vec2f &size) {
			m_Layout.size.minSize = size;
			return this;
		}

		View *body() override {
			auto m_Shape = new sf::RectangleShape(sf::Vector2f{m_Layout.size.minSize.x, m_Layout.size.minSize.y});
			m_Shape->setFillColor(m_Color);
			if (m_BorderThickness > 0) {
				m_Shape->setOutlineThickness(m_BorderThickness);
				m_Shape->setOutlineColor(m_BorderColor);
			}
			m_RenderCalls.emplace_back(sf::RenderStates(), m_Shape);
			return nullptr;
		}

		explicit Rectangle(const vec2f &size) {
			m_Layout.size.minSize = size;
		}
	};
}